# 0.8.0

This release is entirely about improvements to random data generators.

## Datetime generation

* Add a new DSL to create datetimes in a variety of formats with a variety of constraints.
  Type into your IDE `Gen.dateTime.` and bring up auto-complete to explore the options.

  The JVM build supports Java 8 time, epoch longs, and `java.util.Date`.<br>
  The JS build currently only supports epoch longs, and `java.util.Date`.

  Some examples:

  ```scala
  import java.time._
  import scala.concurrent.duration._

  val lastTenYears: Gen[ZonedDateTime] =
    Gen.dateTime.fromNowMinusYears(10).untilNow.asZonedDateTime

  val plusMinus200Days: Gen[Instant] =
    Gen.dateTime.aroundNow(200.days).asInstant

  val year201x: Gen[Long] =
    Gen.dateTime
      .fromZDT (ZonedDateTime parse "2010-01-01T00:00:00+11:00[Australia/Sydney]")
      .untilZDT(ZonedDateTime parse "2020-01-01T00:00:00+11:00[Australia/Sydney]")
      .asEpochMs
  ```

* Add `Gen.zoneId`.


## Seeds and determinism

* Remove `Gen#samplesSized` and have `Gen#samples()` optionally accept a gen size.
* Rename `Gen#samples(GenCtx)` to `Gen#samplesUsing(GenCtx)`.
* Remove `Gen.randomSeed` and make `Gen.reseed` non-deterministic.
* Change the following to adjust the seed based on the sample number and thread number:
  * `Gen.setSeed`
  * `Gen.setOptionalSeed`
  * `Gen#withSeed`
  * `Gen#withOptionalSeed`
* Add the following to using a constant seed regardless of the context.
  Example: `Gen.string.withConstSeed(0).samples().take(99).toSet.size == 1`
  * `Gen.setConstSeed`
  * `Gen.setOptionalConstSeed`
  * `Gen#withConstSeed`
  * `Gen#withOptionalConstSeed`
* Add `Gen.setSeedBy(SeedCtx => Long)` and `Gen#withSeedBy(SeedCtx => Long)`.
* Make `Gen.uuid` deterministic (meaning that it depends on the RNG seed).


## Other

* Add `Gen.chooseLong(bound)`.
* Fix `Gen.chooseInt(l, h)` to work with huge ranges.
* Fix `Gen.chooseLong(l, h)` to work with huge ranges.
* Improve `Gen.chooseInt` speed.

